### A Pluto.jl notebook ###
# v0.20.4

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    #! format: off
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
    #! format: on
end

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
using HypertextLiteral, PlutoUI, NLsolve, PlutoPlotly, CSV, Dates, DataFrames, StatsBase, PeriodicalDates

# ╔═╡ a0775632-4992-440b-b4b8-e7edfbf2ce39
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""
# Week 9 - Macroeconomic Policy: the AD/AS/MP Model

## Exercises

**Macroeconomics, ISCTE-IUL**
"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas** 

**April 2025**
"""

# ╔═╡ 55cbc248-6462-4242-b0e6-14ccc26c1777
md"""
---
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""
### Packages used in this notebook
"""

# ╔═╡ 416e4000-a14b-4517-a067-34a2ddaaa639
md"""
!!! note

	To answer some questions in this set of exercises, it will be helpful to recall what the **_correlation coefficient_** between two variables means. You learned this concept in your statistics course. 

	This correlation coefficient varies between $-1$ and $+1$. A value close to $+1$ means that the two variables move in time with high similarity: one goes up, the other goes up as well very closely; and when one goes down, the other follows very closely. A value close to $-1$ also means a strong similarity in their behavior over time; the only difference in this case is that when one goes up, the other follows by going down. If this correlation coefficient is close to $0$, there is no clear relationship between those two variables over time.

	The following figure presents examples of several cases with positive and negative correlation (taken from: Hyndman, R.J., & Athanasopoulos, G. (2021) _Forecasting: principles and practice_, 3rd edition, OTexts: Melbourne)
"""

# ╔═╡ 6716e258-7411-43c0-811d-8884ae76e04c
Resource("https://modernmacro.org/resources/Vivaldo/figures/cross_correlation.png", :width=>750)

# ╔═╡ 8080d79c-e64d-44f6-b949-5b54d5d5f7a4
md"""
---
"""

# ╔═╡ 0bc2eb14-3b5a-4203-82dd-3ef2044a2c93
md"""
## Exercise 1. Divine coincidence
"""

# ╔═╡ 5999a598-0408-44f1-bd33-c4d0fa5a6006
md"""

Olivier Blanchard, a former MIT professor and IMF Chief Economist, wrote this in 2016:

>*"In the benchmark [macroeconomic] model, stabilizing inflation keeps the unemployment rate at the natural rate, and the natural rate in turn is the best rate that can be achieved by policy. Jordí Gali and I have called this proposition the **“divine coincidence”**.*

👉 a) How do we represent the term "divine coincidence" as a set of conditions that describe society's best possible outcome in our model?

"""

# ╔═╡ 5ceac821-7004-4764-b2ee-b9e2aa0f6fdf
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 6f107fa8-4921-4b4b-bbee-6434ab5e9fb3
md"""
👉 b) Do you consider that, i.e., a 10% unemployment rate may be reasonably acceptable as "natural"?
"""

# ╔═╡ 450dec57-18a3-4031-bf36-36a422d0fd34
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ eb74f385-1688-45ea-8ad1-11bce1e48f87
md"""
---
"""

# ╔═╡ c90e721d-f4ca-473c-8a7c-7159d66d81cf
md"""
## Exercise 2. The Fed's dual mandate
"""

# ╔═╡ 4b6af178-725d-4841-8d62-102c561428c6
md"""
>*"In conducting monetary policy, we will remain highly focused on fostering as strong a labor market as possible for the benefit of all Americans. And we will steadfastly seek to achieve a 2 percent inflation rate over time." Chair Jerome Powell, Aug. 27, 2020 speech* [here](https://www.federalreserve.gov/newsevents/speech/powell20200827a.htm)

The sentence above by the Fed's Chair highlights the two main goals of the Fed. What are those two goals?
"""

# ╔═╡ bd22c100-271d-4496-a55f-792f0f92117f
md"""
!!! answer

	Here

"""

# ╔═╡ 058ec5a1-5e74-472e-8892-c0fad163638b
md"""
---
"""

# ╔═╡ 21d4988a-e438-4562-bc8f-a1d847020e25
md"""
## Exercise 3. Why 2% inflation?

"""

# ╔═╡ 8566ec00-329a-48a8-9b75-0b1a7b9ef9eb
Resource("https://modernmacro.org/resources/Vivaldo/figures/2_percent.png", :width=>450)

# ╔═╡ afeda154-0ce5-4e1b-9e12-24420dd72681
md"""

The image above was taken from "Project Syndicate", July 27, 2023. This publication asked four widely respected economists whether 2% was, for central banks and society, a good target for the inflation rate. Michael J. Boskin, a Professor of Economics at Stanford University and a former Chairman of George H. W. Bush’s Council of Economic Advisers, provided this answer: 

>*"Winston Churchill famously quipped that “democracy is the worst form of government except for all the other forms that have been tried.” The same logic applies to advanced-economy central banks’ inflation targets: compared to anything higher or lower (by a non-trivial margin), 2% is likely to be better."*

"""

# ╔═╡ 08203cfc-1ede-432b-b81c-f0a5cbaa5816
md"""
👉 a) Why do central banks have a target level for the inflation rate?
"""

# ╔═╡ 40913213-c3f7-4dd7-9ffa-c1390f80c099
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ c49227b4-af6d-4c5b-a2fb-ffd15b1c84b8
md"""
👉 b) Why a target of 2%?
"""

# ╔═╡ b6e384ad-f8fb-434c-944a-91ac9f729068
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ a429634a-1e14-4ed5-bfd0-109b0f1448a8
md"""
👉 c) The recently elected President of the USA for 2025-2028, Donald Trump, has clarified that he wants to remove the current Fed's Chair, Jerome Powel (appointed by Trump himself in 2017), who finishes his current mandate only in 2026. In a recent interview, Powel said that he would not resign even if political pressure is exerted upon him and that by law, he can not be removed from his mandate (see the image below for further details, taken from [here](https://www.cnbc.com/2024/11/07/powell-trump.html)). Why do you think the current statutes of the Fed grant considerable independence of the Fed from political interference?

"""

# ╔═╡ 7e7af7e3-8b07-4ce7-871e-f31fae3c13a2
Resource("https://modernmacro.org/resources/Vivaldo/figures/Powel_No.png", :width=>550)

# ╔═╡ f15cce59-c216-4c46-904e-bf1fafbdaa07
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ ec759c26-25d0-4554-96ea-665f6b694204
md"""
---
"""

# ╔═╡ c38dbe18-a85a-4960-9c99-674196e0526d
md"""
## Exercise 4. The Fed's response to shocks

"""

# ╔═╡ cb0d6593-5ca4-4afd-bddc-88506c4cf477
md"""
From the [FRB of New York](https://www.newyorkfed.org/markets/reference-rates/effr) we can download the figure below, where we can see the range for the Fed's nominal interest rate target value. The chosen period is from May 2008 to November 2022. The shaded interval show three main types of decisions by the Fed: (i) keep the range unchanged, (ii) small and gradual changes in the range, and (iii) dramatic changes in at least three occasions. 

👉 a) Can we associate different magnitudes of shocks to these policy decisions?

"""

# ╔═╡ 35612933-4d9a-4b88-b40f-3a2eec3a20fa
Resource("https://vivaldomendes.org/images/depot/FFR.png", :width=>800)

# ╔═╡ 6b39a08d-9f0d-4e46-9f63-549d7eb15c05
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 50ddafe5-0bd0-405b-bdc6-c990b0500661
md"""
👉 b) Can we see a "Zero Lower Bound" (ZLB) on nominal interest rates in the figure above? How likely is it to be found over the last fifteen years or so? 
"""

# ╔═╡ d72389b4-b317-4d1a-928a-c002822ed785
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ f20bd50f-8096-4169-b4ed-bd107fa2537a
md"""
---
"""

# ╔═╡ add07024-b7eb-445d-b4f4-6a6757c2ecd3
md"""
## Exercise 5. Macroeconomic equilibrium
"""

# ╔═╡ 8e2203e6-6677-47b9-8032-e9c63c0f41bb
md"""
Consider the four fundamental functions that allow us to analyze the entire functioning of the economy:

$$\begin{align*}
\text{AD}: \quad Y &=m \cdot \overline{A}-m \cdot \phi \cdot(\overline{r}+\lambda \pi) \\[3mm]           
\text{AS}: \quad  \pi &=\pi^{e}+\gamma\left(Y-Y^{P}\right)+\rho \\[3mm]  
\text{MP}: \quad r&=\overline{r}+\lambda \pi \\[3mm]  
\text{Fisher Eq.}: \quad i& =r +\pi
\end{align*}$$

and the following information concerning exogenous variables and parameters:

$$\begin{aligned}
&\overline{A}=7.6 \ , \ m=2.0 \ , \ \phi=0.2 \ , \ \overline{r}=2.0 \ , \ \lambda=0.5 \ , \ \pi^{e}=2.0 \ , \ \gamma= 4.5 \ , \ Y^{P}=14.0 \ , \ \rho=0.
\end{aligned}$$

👉 a) Calculate the short-run equilibrium values for the inflation rate, the level of GDP, the real interest rate, and the nominal interest rate. Represent graphically, involving the AD and AS curves.


"""

# ╔═╡ 0fe4de49-3687-4900-a691-334cca6dd3c6
md"""
Before we answer the questions above, we have yo pass the information about parameters and exogenous variables into the notebook:
"""

# ╔═╡ a77a222e-aba3-4ac6-a5b0-a7551e3ccce4
begin
	
	Ā5 = 7.6 ; λ5 = 0.5 ; m5 = 2.0 ; ϕ5 = 0.2 ;	r̄5 = 2.0 	# Demand-side parameters and exogenous variables	
	
	γ5 = 4.5 ; Yᴾ5 = 14.0 ; πᵉ5 = 2.0 ; ρ5 = 0.0 			# Supply-side parameters and exogenous variables				
end;

# ╔═╡ 74ee6d92-ef34-47ce-8d57-39d41474a11d
md"""
!!! note
	Note that the $\left\{\pi^{e}, \overline{r}, r, i, \rho \right\}$ are rates and are measured in percentage points (e.g., $2.0$ means $2.0\%$), while $\{Y, Y^P , \overline{A}\}$ are measured in trillion dollars. In the solutions below, to avoid doubt, we put the symbol % only at the final step of the solutions.

	This exercise is a continuation of Exercise 5 of the previous week by adding a new curve: the MP curve. The solution will follow the same steps, but it will have to calculate the values of two new variables: the real interest rate and the nominal interest rate.
"""

# ╔═╡ 3aba8dac-7fa0-4193-9f41-ff8d9a49230f
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 40d80d26-e808-43bf-833e-d1bb07b57630
md"""
#### Solution using NLsolve
"""

# ╔═╡ 1b2dba45-7fbc-4658-9a02-1c54511f650a


# ╔═╡ fc550324-0ffd-4039-b92e-67f12b6b8c90
md"""
!!! tip "Answer (a)"

	Here

"""

# ╔═╡ faaa89d4-9543-40a7-9a61-747e17eb3388
md"""
👉 b) The graphical representation of the initial equilibrium is presented in Figure 2 below. Is the economy in a recession or an economic boom? Justify.
"""

# ╔═╡ 7ae85983-ee8e-4dd3-a7bf-474aaf74863c
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 00f73a3a-f9b7-46a6-aca8-5bf340a79280
md"""
👉 c) If there is no demand or supply shock, what will happen to the economy over time?

"""

# ╔═╡ a54b4a9b-a752-454f-af38-87ec1115512a
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ 445f88ea-3c01-4939-a2bc-a1c139213b17
md"""
---
"""

# ╔═╡ 9b54c17b-e835-4478-95ae-a9e2d5de9f59
md"""
## Exercise 6. A positive demand shock
"""

# ╔═╡ 7f7ca57b-5774-4184-8026-32ad0f56cb39
md"""

Suppose the government, to stimulate economic activity, increases its spending on goods and services (G&S) by 0.2 trillion dollars. This decision is considered a demand shock and will have short-run and long-run impacts. To answer the following questions, use the slider `ΔĀ5` and the figures below.

👉 (a) Using the slider `ΔĀ6` and Fig. 3 and 4 below, describe the short-term impact of this shock on GDP, inflation, and the real interest rate.

"""

# ╔═╡ 7e47b3bc-a45c-4d84-9a58-d4f01d6ba99f
md"""
`ΔĀ6 = `$(@bind ΔĀ6 Slider(-1.0:0.05:1.0, default=0.0, show_value=true))
"""

# ╔═╡ 704145a9-ef2e-48ad-bbe2-b6bc750d62fc
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 13b03571-e332-46f7-83d7-4a41fbcca0b5
md"""
👉 (b) What will be the long-run equilibrium that results from this shock? Give the values of GDP, the inflation rate, real and nominal interest rates. To answer these questions, use the slider `self_correction6` and Fig. 5 and 6 below.
"""

# ╔═╡ 0941b1d5-b354-4857-9264-fcd2744e03a6
md"""
!!! hint
	Move the slider `self_correction6` so the new long-term equilibrium will be correctly represented in the figures below.
"""

# ╔═╡ 63fb8be4-f6ce-457b-8d0c-b359f959f5a6
md"""
`self_correction6 = `$(@bind self_correction6 Slider(-4.0:0.5:4.0, default=0.0, show_value=true))
"""

# ╔═╡ 979c128b-76ed-4c28-b3ad-ffbbac481414
md"""
!!! answer "Answer (b)"

	Here
	
"""

# ╔═╡ 1072c4ef-b41f-4de8-a64d-3cdaecf8949d
md"""
👉 (c) Solve for the new long-term equilibrium using the NLsolve method.
"""

# ╔═╡ 1d0765df-9ee8-4e65-a0d1-bc3e54940846
md"""
!!! hint 
	To answer this question, we have to add the `self_correction6` term to the AD curve and the initial shock `ΔĀ5` to the AD curve, previously written in exercise 4.
"""

# ╔═╡ 0252b84b-8411-4145-81ee-3e90892d859f


# ╔═╡ 9cefbcc6-397b-48d9-810b-05937873184a
md"""
!!! answer "Answer (c)"
	
	Here

"""

# ╔═╡ 3aca9c62-a1e3-4478-a8a5-29d19535ad75
md"""
---
"""

# ╔═╡ dcd13560-0f73-4aaa-b59a-b7961d8cdc63
md"""
## Exercise 7. A positive supply shock
"""

# ╔═╡ 7f67d656-bb55-4097-ba49-10140f9b3502
md"""
Consider the same values for the parameters and the exogenous variables of Exercise 5. However, due to a sharp decline in oil prices, there is a positive supply shock such that now we will have $\rho = -1.9$. 

👉 a) Using the slider `ρ7` below, calculate the short-run equilibrium values for the inflation rate, the level of GDP, the real interest rate, and the nominal interest rate resulting from such a shock. Represent graphically, involving the AD/AS and the MP curves.

"""

# ╔═╡ fe1e6cbc-db87-40ac-836d-68069615c133
md"""
!!! hint 
	Move the slider `ρ6` so the new equilibrium will be correctly represented in the figures below.
"""

# ╔═╡ 22fd1017-4015-407d-bd79-188a763e87a4
md"""
`ρ7  = `$(@bind ρ7  Slider(-3.0:0.1:3.0, default=0.0, show_value=true))
"""

# ╔═╡ f21a5ae9-7887-4cf1-83f9-f1f0a0b3d5b3
md"""
!!! tip "Answer (a)"

	Here
	
"""

# ╔═╡ 7beabfcd-9cbb-4eac-8bdb-298b60f541ac
md"""
👉 b) Is the economy in a recession or a boom? Justify.
"""

# ╔═╡ 28052403-f02e-41dc-ba74-e42d88f3d494
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ 70e7a7ef-d49d-417e-8f87-ed9870857b45
md"""
👉 c) If there are no other supply or demand shocks, what will happen to the economy over time? Answer this question by manipulating the slider `self_correction7` below.
"""

# ╔═╡ db702f28-e3e7-4d14-9209-1cbe2917247b
md"""
`self_correction7 = `$(@bind self_correction7 Slider(-4.0:0.1:4.0, default=0.0, show_value=true))
"""

# ╔═╡ 5859e1fa-f4ef-42b7-a3fd-5085cd64161c
md"""
!!! tip "Answer (c)"

	Here

"""

# ╔═╡ 933a7956-1037-401e-a0cd-e42ec8eeec2d
md"""
---
"""

# ╔═╡ ec34cbc5-fda5-4792-adc8-3bb0d02a7f22
md"""
## Exercise 8. A permanent postive supply shock
"""

# ╔═╡ 83c030a3-28bc-419f-968b-62ce9ab0a1e4
md"""
Consider the same data as in Exercise 5. However, suppose the supply shock is **permanent** instead of temporary (like in the previous exercise); oil prices go down, and then they will stay there for an extended period. This positive shock will cause Potential GDP to increase by 0.2 trillion dollars. 

How would we change our answers to the three questions raised in the previous exercise? Use the slider `self_correction8` to answer this exercise.
"""

# ╔═╡ ab9daba1-be70-4e7f-8304-639db1c16d2c
md"""
`self_correction8 = `$(@bind self_correction8 Slider(-4.0:0.1:4.0, default=0.0, show_value=true))
"""

# ╔═╡ 79d2fb58-371b-4720-976c-d5f6a70e46c6
md"""
!!! answer

	Here

"""

# ╔═╡ a8bfdb49-6a0a-448a-b84e-b5040f8a40c9
md"""
---
"""

# ╔═╡ c20a2418-ef6f-486d-a88b-6345655bb449
md"""
## Exercise 9. Fiscal Policy Shocks and Real GDP 
"""

# ╔═╡ 7c7b66f0-2136-49bf-9aa5-829d5a342549
md"""

If a government introduces legislation that increases taxes or reduces spending in one particular year, those decisions will usually have some impact in the coming years. But for how long? And for how much? The Fed (Federal Reserve Board, the US central bank) and the Hutchins Center developed a technique that intends to answer those questions.

>*"The Hutchins Center Fiscal Impact Measure shows how much local, state, and federal tax and spending policy adds to or subtracts from overall economic growth, and provides a near-term forecast of fiscal policies’ effects on economic activity."* [here](https://www.brookings.edu/articles/hutchins-center-fiscal-impact-measure/)

In the figure below, we present evidence of fiscal policy decisions' impact on US GDP.

"""

# ╔═╡ 786fb701-2d51-4257-a474-39779b6b98e5
md"""
👉 **a)** By inspecting the figure above, provide evidence of two periods when the US government stimulated economic activity.
"""

# ╔═╡ 07f23f65-74e4-4a0f-aa18-ad252c25035f
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 56570fa6-7445-4858-a0be-845ef4a50510
md"""
👉 **b)** By inspecting the figure above, provide evidence of two periods when the US government cooled off (negative stimulus) economic activity.
"""

# ╔═╡ e6542ffd-3ca5-4bf2-a9e8-01bf8b5f1f17
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ b8fe25b2-7995-4394-9b54-d515493525ec
md"""
---
"""

# ╔═╡ 6cf60fca-ee40-4036-9bdc-4c080983658b
md"""
## Exercise 10. Supply shocks, what shocks?
"""

# ╔═╡ 909376d1-fe62-4823-8a03-916147ec6da3
md"""

In the post-COVID period, Western economies suffered a burst in inflation that had not been seen since the early 1980s (more than forty years ago). This burst came as a surprise and remains an enigma. What forces caused such a big surge in inflation? There are still no convincing answers about those forces. President Biden's Council of Economic Advisors has been portraying an idea that negative **_supply chain shocks_** are mainly responsible for that surge. For example, in a recent document, we can read:

>*"In the transportation and logistics industry specifically, a series of supply chain disruptions and delays at ports during the pandemic led to historically high prices for imports to the United States. At their highest point, spot shipping prices for containers from China to U.S. West Coast ports skyrocketed to more than 1000 percent of 2019 levels ... The correlation between supply chains and inflation can be seen in the Global Supply Chain Pressure Index [GSCPI], produced by the Federal Reserve Bank of New York".* [here](https://www.whitehouse.gov/cea/written-materials/2023/11/30/issue-brief-supply-chain-resilience/#:~:text=During%20the%20global%20pandemic%2C%20it,shocks%20that%20drove%20up%20prices.)

The figure below presents two cross-plots: one between "headline" inflation and the GSCPI indicator (left panel) and the other between "headline" inflation and oil prices (right panel). Both panels cover the same period as the paper mentioned above: pre-COVID (2016-2020) and post-COVID (2020-2024). 

"""

# ╔═╡ 7695e6fb-92b5-4a8d-aea3-a4c9fb67bf8f
md"""
!!! note "What is "headline" inflation?"

	In this exercise, a term needs further clarification: "headline inflation". Central banks usually use two different types of indicators for inflation: "headline" and "core" inflation. Headline inflation is the rate of inflation that includes the prices of all goods and services consumed (or a representative basket of them), while "core" inflation excludes the prices of food and energy goods.
"""

# ╔═╡ 92205224-29aa-4318-9f5d-6170a5d714d8
md"""
👉 **a)** The correlation coefficient between those macroeconomic variables are as follows:

- (Headline) inflation vs. GSCPI index: 0.48
- (Headline) inflation vs. Oil prices: 0.85

Given this information and the theory we learned about inflation in our course, the causality amongst these variables runs in what direction: (i) inflation causes GSCPI (or vice-versa), (ii) inflation causes oil prices (or vice versa), (iii) despite positive correlations, there is no causality between those variables.
"""

# ╔═╡ 1a322c48-9310-4850-826f-be458ea6de84
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 4d52d8da-73fa-4001-988e-b50a01081295
md"""
👉 **b)** Which supply shock (oil prices or supply chains) do you find more convincing to explain the big US inflation surge in the post-COVID period?
"""

# ╔═╡ 2052e846-8d58-46f5-89d5-1471929c9b4e
md"""
!!! note

	You can use the mouse to hover over the points to follow the sequence of dates in both plots. By doing that, you will find out that the highest values for inflation are in the post-Covid period.

"""

# ╔═╡ 3d1d85f7-f000-40bb-ad7a-6aaace47d8f5
md"""
!!! answer "Answer(b)"

	Here

"""

# ╔═╡ bbc0b205-f298-41e6-b216-3177389815e3
md"""
---
"""

# ╔═╡ 765bcd30-07e8-4a38-ae79-30f513bbfc49
md"""
## Exercise 11. The Sahm rule: recession on your computer?
"""

# ╔═╡ 27ec69d5-a112-48fb-ac46-9673f06d7322
md"""

There is currently a rule for telling whether an economy is (or is not) in a recession -- the Sahm rule -- that has become very famous because it **_never failed in the past_** (since we have data: the II World War). One department of the US Congress recently (August 26) published a note discussing the application of this rule to the US economy. From this note, we can read: [here](https://crsreports.congress.gov/product/pdf/IN/IN12410)

>*"Economist Claudia Sahm created a real-time indicator in 2019 that is used by many economists and policymakers to identify whether the economy may be in a recession. The Sahm rule is triggered when the three-month moving average of the unemployment rate increases by 0.5 percentage points or more relative to its low in the previous 12 months. The Sahm rule has typically been triggered in the beginning months of a recession."*

The title of the Congress publication is "The Sahm Rule Trigger: Is the United States
in a Recession?" By inspecting the figure below, what is your opinion about whether the US economy is on a path to recession or not?

"""

# ╔═╡ 2f6baaac-a510-452e-b8e0-b84294d39aae
md"""
👉 **a)** By inspecting the figure above and taking into consideration the information that the Sahm rule provides, what is your opinion about the current state of the US economy: (i) it is more likely to fall into a recession, (ii) it is more likely to go into a boom, (iii) neither of them.
"""

# ╔═╡ 8dd55cb9-c3b5-4be6-b490-5f50a8d3250e
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 84541347-f1fb-42b6-ba7b-74712c6def7d
md"""

👉 **b)** Suppose you are an advisor to a member of the Fed who sits and votes on its board. The board will meet three times until the end of 2024. What would be your advice in early September 2024? 

"""

# ╔═╡ 6c1dbd2e-5bcf-4cc3-9309-e305f22d9260
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ e9d17d76-4bb2-4395-b56e-991c00a8d5c5
md"""

👉 **c)** Reuters conducts a poll of 101 economists who, every month, try to predict the Fed's decisions regarding interest rates. This poll can be found [here](https://fingfx.thomsonreuters.com/gfx/polling/lgvdodmqapo/Reuters%20poll%20-%20U.S.%20economy%20and%20Fed%20monetary%20policy%20-%20September%202024.png). On September 11, 2024, Reuters communicated its results for this month poll, and it says [here](https://www.reuters.com/markets/us/fed-cut-rates-by-25-basis-points-sept-18-twice-more-2024-2024-09-10/):


>*"The Federal Reserve will lower interest rates by 25 basis points at each of the U.S. central bank's three remaining policy meetings in 2024, according to a majority of economists in a Reuters poll that found only nine of 101 expected a half-percentage-point cut next week."*

However, on September 18th, the Fed cut interest rates by 0.5 percentage points (50 basis points, not 25). Why did the forecast made on September 11th prove so wrong about the Fed's decision a week later?
"""

# ╔═╡ d7527a4a-a4e4-40e3-8bb0-3dda5c133aa7
md"""
!!! answer "Suggested answer (c)"

	Here
	
	"""

# ╔═╡ 867af7b9-5847-4b56-8a98-5995c3da8159
md"""
---
"""

# ╔═╡ f7853e38-66f3-457e-951f-01e04cda9ba0
md"""
## Exercise 12. Policy coordination, or common bad luck?
"""

# ╔═╡ 08fe04ec-464e-4d1a-a0ca-84125903f1d5
md"""
The figure below plots the nominal interest rates set by four major central banks in the world economy: the US, UK, Canada, and the EuroZone (ECB). What is most striking is that these four central banks changed interest rates in tandem during this period (2005-2024). 

👉 **a)** Describe what happened in four periods: (i) the Great Recession, (ii) the Covid pandemic, (iii) the recent rampant inflation, and (iv) between 2010 and 2016.

"""

# ╔═╡ aaa23489-2459-44ef-acd4-cbf25ae1f7a3
md"""
!!! answer "Answer (a)"

	- During the Great Recession, ...

	- During Covid, ...

	- During rampant inflation, ...

	- Between 2010 and 2016, ...

"""

# ╔═╡ 130e1429-87a6-4ead-aa31-3d8ecbea91e5
md"""
👉 **b)** Are the similar reactions by those central banks a result of policy coordination among themselves?
"""

# ╔═╡ 0d7a7c6d-5a53-4c00-a726-0eabf26d0edd
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ e35df418-41e9-4f78-97fc-f427f588fb1e
md"""
👉 **c)** In the plot below, we present the evolution of headline inflation in those four economic areas. Do you find support for the view that central banks make similar decisions over time because they face similar shocks?
"""

# ╔═╡ 1e427799-fcd1-4cb5-a1a6-a5c21470e515
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 468f94f8-a10c-4ea3-b795-1ea4f2f2a724
md"""
---
"""

# ╔═╡ df3e386a-66f1-4525-b09d-a605c24aff21
md"""
## Appendix 1. More on the natural unemployment rate
"""

# ╔═╡ 4a1e9746-51d3-4981-b685-2d1545fb2a07
md"""
!!! note

	This Appendix is considered to be of interest only to those students who have a particular interest in macroeconomic controversies.

"""

# ╔═╡ 15c7a983-f8d3-44a5-a08a-7426dcee7639
md"""
Do you consider that, i.e., a 10% unemployment rate may be reasonably acceptable as "natural"?
"""

# ╔═╡ e1ad4f38-80cf-4730-9f5e-1a0c021becff
md"""
!!! answer

	It depends; it turns out to be more a political opinion than a scientific one. We have two basic types of unemployment: natural unemployment (=structural + frictional) and cyclical unemployment (which depends on the cycles of economic activity). Macroeconomic policy can affect cyclical unemployment (everybody agrees) but can do little to affect structural and frictional unemployment (some agree, some disagree).

	Why the disagreement? Because people have political views, putting them behind "structural unemployment" is possible. For example, Narayana Kocherlakota (a former President of the Federal Reserve Bank of Minneapolis and Professor at Stanford University) made a famous speech in 2010, [Inside the FOMC:](https://www.minneapolisfed.org/speeches/2010/inside-the-fomc)

	>*"What does this change in the relationship between job openings and unemployment connote? In a word, mismatch. Firms have jobs but can’t find appropriate workers. The workers want to work but can’t find appropriate jobs. [...] Of course, the key question is: How much of the current unemployment rate is due to mismatch, as opposed to conditions the Fed can readily ameliorate? The answer **seems to be a lot**. I mentioned that the relationship between unemployment and job openings was stable from December 2000 through June 2008. If that stable relationship were still in place today, and given the current job opening rate of 2.2 percent, we would have an unemployment rate closer to 6.5 percent, not 9.5 percent. **Most of the existing unemployment represents mismatch that is not readily amenable to monetary policy**."*(our emphasis)

	This view is defended by Classical economists, who stress that economic policy can do very little to fight structural unemployment. Keynesian economists question such a view because they argue that empirical evidence can easily refute the Classical argument. In the US, looking at the figure below, how is it possible that in just ten years, the unemployment rate was 4.4% in mid-2007 (in the middle of a downward process), jumped to 10% in October 2009, to go back to 4.4% in May 2017? The Keynesian question is: "What changed so structurally in the labor market throughout those ten years to explain that behavior of unemployment in the US economy?"

	The two views are antagonistic, and in macroeconomics, it is vital to understand the logic behind each one before choosing sides. Therefore, considering the initial question, for the Classicals, an unemployment rate of 10% can be perfectly acceptable as "natural," and the Fed can do nothing to change it. At the same time, there is nothing "natural" in such a number for the Keynesians, but rather a sign of a drastic collapse in aggregate demand, which the central bank policy can quickly reverse.


"""

# ╔═╡ 34f82c93-1bad-4635-8e5f-3e9960657429
Resource("http://ebs.de.iscte.pt/UnemploymentUS.png", :width=>900)

# ╔═╡ ca679141-76e6-4b59-bbcb-ecfb923e8765
md"""
---
"""

# ╔═╡ de32bd19-9ef4-45f2-96cd-3a9675d9527d
md"""
## Auxiliary cells (do not delete)
"""

# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ cc156e07-e234-4f21-ac6e-ddaff41b3579
md"""
##### Data files used in this notebook

- GlobalSupplyChainIndex.csv
- FFR\_CPI\_Oil.csv
- Sahm_Rule.csv
- NBER_Recessions.csv
- CentralBankRates.csv
- Hutchins-fiscal-impact.cvs

These CSV files must be kept in the same folder as this notebook.
"""

# ╔═╡ f842d504-2d12-4fcc-966c-fdb6416d1ec5
md"### CSS code"

# ╔═╡ 6d02586e-d1a4-4628-8c81-339db95b898e
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1560px;
    		padding-left: max(100px, 10%);
    		padding-right: max(380px, 10%); 
            # 383px to accomodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ 60e502e1-3990-49a2-b8f6-8f2ad744bb01
md"### Exercises"

# ╔═╡ 45c90777-16b2-4359-9afa-31fdaf23fd73
md"""
##### Exercise 5. Supporting cells (Macroeconomic equilibrium)
"""

# ╔═╡ 99b637d3-af40-4132-aa5b-d0d386b8d11a
begin
	πd_max5 = (Ā5 / (λ5 * ϕ5)) - r̄5/λ5       							 # The AD curve y-axis' intercept
	
	πs_min5 = πᵉ5 - γ5 * Yᴾ5 + ρ5           							  # The AS curve y-axis' intercept
end;

# ╔═╡ 0ed308d6-1457-4571-9898-d6fe23ca827f
begin
	Y5 = 12.0:0.01:15.0
	
	πd5 = πd_max5 .- ((1 ./(m5 .* ϕ5 .* λ5)) .* Y5) # AD function
	πs5 = πs_min5 .+ γ5 .*Y5             # AS function
	
	trace9_0 = scatter(; x = [14,14] , y = [-7,14] , mode = "line", line_width = "4", name = "Yp", 
					line_color = "Orange")
	
	trace9_1 = scatter(; x = Y5, y = πd5, mode="lines" , line_color = "Blue", line_width = "3",
			name  = "AD")
	
	trace9_2 = scatter(; x = Y5, y = πs5, mode="lines" , line_color = "Red", line_width = "3",
					name  = "AS")
	
	trace9_3 = scatter(; x = [14.0], y = [2.0], text =["1"], 
					textposition = "top center", name ="Eq1", mode="markers+text", marker_size= "12",
					marker_color="Blue", textcolor = "Black",
	 				textfont = attr(family="sans serif", size=18, color="black"))
	
	
	layout9_3 = Layout(; height = 450,
					title_text = "Fig 2. Macroeconomic Equilibrium",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                    xaxis_range = [13.5, 14.5],	
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-0.5 , 4.5])

	p9_3 = plot([trace9_0, trace9_1, trace9_2, trace9_3], layout9_3)
	
end;

# ╔═╡ 1f24f76d-6f4e-4031-8d62-41b0ec351a2c
begin
	#function fig9_2()
		r5 = 0.0:0.01:5.0
	
		πmp5 = - (r̄5 ./ λ5) .+  r5 ./ λ5 
		
		trace4_0 = scatter(; x = r5, y = πmp5, mode="lines" , line_color = "Brown", line_width = "3",
				name  = "MP")
		
		trace4_1 = scatter(; x = [3.0], y = [2.0], text =["1"], 
						textposition = "top center", name ="Eq1", mode="markers+text", 
						marker_size= "12", marker_color="Brown", 
						textfont = attr(family="sans serif", size=18, color="black"))
		
		layout4_1 = Layout(; height = 450,
						title_text = "Fig 1. MP curve",
						title_x = 0.5,
						hovermode = "x",
	                    xaxis=attr(title=" Real interest rate (r)", showgrid=true, zeroline=false),
	                    xaxis_range = [1.6, 4.2],
						xaxis_tickvals = [2, 2.5, 3, 3.5, 4, 4.5],
	                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
						yaxis_range=[-0.5 , 4.5])
	
	   p4_1 = plot([trace4_0, trace4_1], layout4_1)
	#end
end;

# ╔═╡ 6e92b539-a91d-410f-a895-8d4b51b5257b
begin
	p02 =[p4_1 p9_3]
	relayout!(p02, height = 460)
	p02
end

# ╔═╡ 62b870d7-80a7-46ea-b784-579d5ad289c2
md"""
##### Exercise 6. Supporting cells (Demand shock)
"""

# ╔═╡ 43e8fd65-2d3a-41bc-a8d6-44cf412318ed
begin
	Y5_ΔĀ6 = 12.0:0.02:15.0
	
	πd_max5_ΔĀ6 = ((Ā5 .+ ΔĀ6) ./ (λ5 .* ϕ5)) .- r̄5 ./ λ5 # New max value of the AD function
	
	πd5_ΔĀ6 = πd_max5_ΔĀ6 .- ((1 ./(m5 .* ϕ5 .* λ5)) .* Y5_ΔĀ6) # AD function
	
	trace9_4 = scatter(; x = Y5_ΔĀ6, y = πd5_ΔĀ6, mode="lines" , line_color = "Magenta", 
					 line_width = "3", name  = "AD2")
		
	trace9_5 = scatter(; x = [14.21], y = [2.947], text =["2"], 
					textposition = "top center", name ="Eq2", mode="markers+text", 
					marker_size= "12", marker_color="Magenta",
					textfont = attr(family="sans serif", size=16, color="black"))
	
	
	layout9_5 = Layout(; height = 450,
					title_text ="Fig 4. A Positive Demand Shock",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                    xaxis_range = [13.6, 14.5],	
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-0 , 5])

   p9_4 = plot([trace9_0, trace9_1, trace9_2, trace9_4, trace9_3, trace9_5], layout9_5)
	
end;

# ╔═╡ 955e1066-8468-4553-b834-f21116d4a594
begin
	
	trace4_3 = scatter(; x = [3.473], y = [2.946], text =["2"], 
					textposition = "top center", name ="Eq2", mode="markers+text", 
					marker_size= "12", marker_color="Brown", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout4_3 = Layout(; height = 450,
					title_text ="Fig 3. MP curve",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title= "Real interest rate (r)", showgrid=true, zeroline=false),
                    xaxis_range = [2.0, 4.5],
					xaxis_tickvals = [2, 2.5, 3, 3.5, 4, 4.5],
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-0 , 5])
	
	p4_3 = plot([trace4_0, trace4_1,trace4_3], layout4_3)
end;

# ╔═╡ 5c2ce460-bd7a-4355-8b11-d8e6896d973d
begin
	p03 =[p4_3 p9_4]
	relayout!(p03, height = 460)
	p03
end

# ╔═╡ df2a5996-55a7-4883-ac2f-59d4cce2c129
begin
	Y5_self = 12.0:0.01:15.0
	
	πs5_self = πs_min5 .+ γ5 .*Y5_self .+ self_correction6            # AS function
	
	trace9_6 = scatter(; x = Y5_self, y = πs5_self, mode="lines" , line_color = "Coral", line_width = "3",
			name  = "AS2")	
	
	trace9_7 = scatter(; x = [14.0], y = [4.0], text =["3"], textposition = "top center", 
					name ="Eq3", mode="markers+text", marker_size= "12", marker_color="Coral", 
					textfont = attr(family="sans serif", size=18, color="black"))
	
	
	layout9_6 = Layout(; height = 450,
					title_text = "Fig 6. A Positive Demand Shock",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                    xaxis_range = [13.6, 14.5],	
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[1 , 5])
	

    p9_6 = plot([trace9_0,trace9_1,trace9_2,trace9_4, trace9_6, trace9_3, trace9_5,trace9_7], layout9_6)
	
end;

# ╔═╡ e1fc276a-db60-4de2-b21e-4e6bfcdbe6e2
begin
	
	trace4_4 = scatter(; x = [4.0], y = [4.0], text =["3"], 
					textposition = "top center", name ="Eq3", mode="markers+text", 
					marker_size= "12", marker_color="Brown", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout4_4 = Layout(;height = 450,
					title_text ="Fig 5. MP curve",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" Real interest rate (r)", showgrid=true, zeroline=false),
                    xaxis_range = [2.5, 4.5],
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[1 , 5])
	
	p4_4 = plot([trace4_0, trace4_1, trace4_3, trace4_4], layout4_4)
end;

# ╔═╡ 4b11d361-dab6-4440-b269-499b36737f0d
begin
	p04 =[p4_4 p9_6]
	relayout!(p04, height = 460)
	p04
end

# ╔═╡ f029f20a-d065-4211-b71f-18c56bc41a7a
md"""
##### Exercise 7. Supporting cells (Supply shock)
"""

# ╔═╡ a1b1866b-3179-4829-a0cf-9bdbdb913c8c
begin
	Y6 = 12.0:0.02:15.0
	
	πs6 = πs_min5 .+ γ5 .*Y6 .+ ρ7            # AS function
	
	trace9_8 = scatter(; x = Y6, y = πs6, mode="lines" , line_color = "Coral", line_width = "3",
			name  = "AS2")	
	
	trace9_9 = scatter(; x = [14.2], y = [1.0], text =["2"], 
					textposition = "top center", name ="Eq2", mode="markers+text", 
					marker_size= "12", marker_color="Coral",
					textfont = attr(family="sans serif", size=16, color="black"))
	
	
	layout9_9= Layout(; height = 450,
					title_text ="Fig 8. A Positive Temporary Supply Shock",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                    xaxis_range = [13.6, 14.5],	
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-1 , 4])
	

    p9_9 = plot([trace9_0,trace9_1,trace9_2, trace9_8, trace9_3, trace9_9], layout9_9)
	
end;

# ╔═╡ 2d8d028a-2709-4ba4-bf95-cf927187c67a
begin
	
	trace4_5 = scatter(; x = [2.5], y = [1.0], text =["2"], 
					textposition = "top center", name ="Eq2", mode="markers+text", 
					marker_size= "12", marker_color="Brown", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout4_5 = Layout(; height = 450,
					title_text ="Fig 7. MP curve",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" Real interest rate (r)", showgrid=true, zeroline=false),
                    xaxis_range = [1.5, 4.0],
					xaxis_tickvals = [1.5, 2, 2.5, 3, 3.5, 4],
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-1 , 4])
	
	p4_5 = plot([trace4_0, trace4_1, trace4_5], layout4_5)
end;

# ╔═╡ 1d36abf0-a3ea-45d1-9ef2-1a8a1fdc50b4
begin
	p05 =[p4_5 p9_9]
	relayout!(p05, height = 460)
	p05
end

# ╔═╡ 2b1953d1-4bb8-42c6-98be-c353a44d8857
begin
	Y6_self6 = 12.0:0.01:15.0
	
	πs6_self6 = πs_min5 .+ γ5 .*Y6_self6 .+ ρ7   .+ self_correction7           # AS function
	
	trace9_10 = scatter(; x = Y6_self6, y = πs6_self6, mode="lines" , 
		line_color = "burlywood", line_width = "3",	name  = "AS2")	
	
	trace9_11 = scatter(; x = [14.0], y = [2.0], text =["3"], 
					textposition = "bottom center", name ="Eq3", mode="markers+text", 
					marker_size= "12", marker_color="burlywood", 
					textfont = attr(family="sans serif", size=18, color="black"))
	
	layout9_10= Layout(; height = 450,
					title_text ="Fig 10. A Positive Temporary Supply Shock",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                    xaxis_range = [13.6, 14.5],	
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-1 , 4])
	
	

    p9_11 = plot([trace9_0,trace9_1,trace9_2, trace9_10, trace9_3, trace9_9, trace9_11], layout9_10)
	
end;

# ╔═╡ c435e0de-1b92-4062-a30e-a1da56a91914
begin
	trace4_6 = scatter(; x = [3.0], y = [2.0], text =["3"], 
					textposition = "bottom center", name ="Eq3", mode="markers+text", 
					marker_size= "12", marker_color="Brown", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout4_6= Layout(; height = 450,
					title_text ="Fig 9. MP curve",
					title_x = 0.5,
					hovermode = "x",
                     xaxis=attr(title=" Real interest rate (r)", showgrid=true, zeroline=false),
                     xaxis_range = [1.5 , 4.0],
					 xaxis_tickvals = [1.5, 2, 2.5, 3, 3.5, 4],
                     yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					 yaxis_range=[-1 , 4])
	
	p4_6 = plot([trace4_0, trace4_1, trace4_5, trace4_6], layout4_6)
end;

# ╔═╡ 39b9f0db-8735-4d0d-8e75-384bf21c1bc8
begin
	p06 =[p4_6 p9_11]
	relayout!(p06, height = 460)
	p06
end

# ╔═╡ 01ce076b-3033-4f1b-acb7-1f775c4ed1e1
md"""
#####  Exercise 8. Supporting cells (Permanent shock)
"""

# ╔═╡ f760841b-8a69-4aa4-a76b-edbf27f7de63
begin
	Y7 = 12.0:0.01:15.0
	
	trace9_12 = scatter(; x = [14.2,14.2] , y = [-7,14] , mode = "line", line_width = "4", name = "Yp2", 
					line_color = "Orange")
	
	πs7 = πs_min5 .+ γ5 .*Y5 .+ ρ5  .+ self_correction8           # AS function
	
	trace9_13 = scatter(; x = Y7, y = πs7, mode="lines" , line_color = "burlywood", 
					line_width = "3", name  = "AS2")	
	
	layout9_13= Layout(; height = 450,
					title_text ="Fig 12. A Positive Permanent Supply Shock",
					title_x = 0.5,
					hovermode = "x",
                     xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                     xaxis_range = [13.6, 14.5],	
                     yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					 yaxis_range=[-1.0 , 4.0])
	

    p9_13 = plot([trace9_0, trace9_12, trace9_1,trace9_2, trace9_13, trace9_3, trace9_9], layout9_13)
	
end;

# ╔═╡ 88491aa6-3a00-4237-b8a7-a1385b69eccf
begin
	
	trace4_7 = scatter(; x = [2.5], y = [1.0], text =["2"], 
					textposition = "top center", name ="Eq.", mode="markers+text", marker_size= "12",
					marker_color="Brown", textcolor = "Black")
	
	layout4_7 = Layout(; height = 450,
					title_text ="Fig 10. MP curve",
					title_x = 0.5,
					hovermode = "x",
                     xaxis=attr(title=" Real interest rate (r)", showgrid=true, zeroline=false),
                     xaxis_range = [1.5, 4.0],
					 xaxis_tickvals = [1.5, 2, 2.5, 3, 3.5, 4],
                     yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					 yaxis_range=[-1 , 4])
	
	p4_7 = plot([trace4_0, trace4_1, trace4_5], layout4_7)
end;

# ╔═╡ f331c8bc-ab24-4547-b9e9-f1d543916862
begin
	p07 =[p4_7 p9_13]
	relayout!(p07, height = 460)
	p07
end

# ╔═╡ 659c291b-ec4a-46d8-90ed-73fd3ce3b7b9
md"""
##### Supporting cells for exercise 9 (Fiscal policy shocks)
"""

# ╔═╡ c01915e5-2cab-4760-b9a9-6e7bf4fed84b
hutchins=CSV.read("Hutchins-fiscal-impact.csv", DataFrame);

# ╔═╡ 992942ea-85b0-480d-a872-ba668092f55c
let
    
    # Define the trace for the bar chart
    trace2_30 = bar(; x=hutchins.quarter, y=hutchins.total,
                    name="Spread", mode="markers+lines",
                    marker_symbol="circle", marker_size=5, line_width=0.3,
                    marker_color="Blue")

    # Define the layout
    layout2_30 = Layout(;
        title_text="Contribution of Fiscal Policy to Real GDP Growth",
        title_x=0.5,
        hovermode="x",
        xaxis=attr(
            title="Quarterly observations",
            tickformat="%Y",
            hoverformat="%Y-Q%q",
        ),
        yaxis_title="Percentage points",
        titlefont_size=16
    )

    # Create the plot with the defined trace and layout
    p2_30 = plot([trace2_30], layout2_30)

    # Define the annotations correctly as a list of dictionaries
    annotations = [
        attr(x="2002-12-01", y=12, text="Recession", showarrow=false, yshift=10),
        attr(x="2010-06-01", y=12, text="Recession", showarrow=false, yshift=10),
        attr(x="2019-06-01", y=12, text="Recession", showarrow=false, yshift=10)
    ]

    # Relayout the plot with annotations and set the figure height
    relayout!(p2_30, height=450, annotations=annotations)

    # Add vertical rectangles to highlight recession periods
    add_vrect!(p2_30, "2001-04-01", "2002-01-01", fillcolor="red", opacity=0.25, line_width=0; row=1, col=1)
    add_vrect!(p2_30, "2008-01-01", "2009-07-01", fillcolor="red", opacity=0.25, line_width=0; row=1, col=1)
    add_vrect!(p2_30, "2020-05-01", "2020-10-01", fillcolor="red", opacity=0.25, line_width=0; row=1, col=1)

    # Display the plot
    p2_30
end

# ╔═╡ 920ce92a-437c-4b23-b56b-bf6d810329ba
md"""
##### Supporting cells for exercise 10 (Supply shocks shocks)
"""

# ╔═╡ bb3bb4b1-7bb3-4bfd-aeaf-b11703c050e8
supply_index = CSV.read("GlobalSupplyChainIndex.csv", DataFrame);

# ╔═╡ 0aaafde6-ada0-45ab-a684-3593d6e8dc10
cor(supply_index.CPI[217:end] , supply_index.GSCPI[217:end]);

# ╔═╡ b21af848-8878-4de6-8a96-4221962a8fb5
cor(supply_index.CPI[217:end] , supply_index.OilPrices[217:end]);

# ╔═╡ ac39db85-3a1e-4f49-99f9-d12153545e71
infl=CSV.read("FFR_CPI_Oil.csv", DataFrame);

# ╔═╡ d9d9266f-e090-4462-b535-634bcb311365
begin
    #using DataFrames, Dates, PlotlyJS

    # Example data creation (Replace with your actual data)
    mydata3 = infl 
	mydata_GSCPI = supply_index


    # Define the subperiod you want to filter
    start_date3 = Date("2016-01-01")
    end_date3   = Date("2024-08-01")

    # Filter the DataFrame for the desired date range
    filtered_data3 = mydata3[(mydata3.Months .>= start_date3) .& (mydata3.Months .<= end_date3), :]
	filtered_GSCPI = mydata_GSCPI[(mydata_GSCPI.Months .>= start_date3) .& (mydata_GSCPI.Months .<= end_date3), :]

    # Plotting the filtered data
    trace_inflation3 = scatter(; 
        x=filtered_data3.Months, 
        y=filtered_data3.CPIHeadline, 
        mode="lines", 
        line_color="blue", 
        name="Inflation (Headline CPI)",
		yaxis="y"  # Assign to primary y-axis
    )

    trace_oil_prices3 = scatter(; 
        x=filtered_data3.Months, 
        y=filtered_data3.OilPrices, 
        mode="lines", 
        line_color="purple", 
        name="Oil prices",
        yaxis="y2",  # Assign to secondary y-axis
    )

	trace_GSCPI = scatter(; 
        x=filtered_GSCPI.Months, 
        y=filtered_GSCPI.GSCPI, 
        mode="lines", 
        line_color="DarkOrange", 
        name="Global Supply Chain Price Index",
        yaxis="y1",  # Assign to secondary y-axis
    )

    # Combine the traces into a figure
    fig3 = plot([trace_inflation3, trace_oil_prices3, trace_GSCPI])

    # Customize layout with dual y-axes
    relayout!(fig3,
		height = 450,
        title_text="Headline Inflation vs Growth Rate of Oil Prices (1970-1990)",
        title_x=0.5,
        hovermode="x unified",
		legend=attr( x=0.02, y=0.97),
        xaxis=attr(title="Date", showgrid=true, zeroline=false, dtick="M24"),
        yaxis=attr(title="Inflation (Left)", zeroline=false, showgrid=true),
        yaxis2=attr(
            title="Oil Prices (Right)", 
            overlaying="y", 
            side="right", 
            showgrid=false
        )
		

    )
	
	#add_vrect!(fig2, "1979-09-01", "1987-09-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)

    # Display the figure
    fig3
end;

# ╔═╡ 18cb3039-8cc6-4faa-ba5e-f8c6bca82318
begin
	period1 = MonthlyDate(2016, 1):Month(1):MonthlyDate(2024, 8);
	p33 = plot(supply_index.CPI[217:end], supply_index.GSCPI[217:end], mode = "markers+lines", line_width = 0.3,
		marker_color = "darkred", marker_size=7, text=period1
	)
	
	relayout!(p33, height = 500, width = 500, title_text = "Headline inflation vs. GSCPI (2016-2024)", xaxis_title = "Headline inflation", yaxis_title = "GSCPI")
	
	p33
end;

# ╔═╡ f1e64c63-b2eb-4ac6-8306-0801ccb77e24
begin
	#period1 = MonthlyDate(2016, 1):Month(1):MonthlyDate(2024, 8);
	p53 = plot(supply_index.CPI[217:end], supply_index.OilPrices[217:end], mode = "markers+lines", line_width = 0.3,
		marker_color = "darkred", marker_size=7, text=period1
	)
	
	relayout!(p53, height = 500, width = 500, title_text = "Oil prices vs. GSCPI (2016-2024)", xaxis_title = "Headline inflation", yaxis_title = "Oil prices")
	
	p53
end;

# ╔═╡ 48899ef5-f238-4a43-8589-1e0379a35a39
begin
	p56=[p33 p53]
	restyle!(p56, showlegend= false, marker_color = ["blue" ,  "darkred"])
	relayout!(p56, height = 450)
	p56
end

# ╔═╡ eaf16e5b-4008-4bfe-8b00-2148ae964d61
begin

    # Example data creation (Replace with your actual data)
    mydata5 = infl

    # Define the subperiod you want to filter
    start_date5 = Date("2000-01-01")
    end_date5   = Date("2024-08-01")

    # Filter the DataFrame for the desired date range
    filtered_data5 = mydata5[(mydata5.Months .>= start_date5) .& (mydata5.Months .<= end_date5), :]

    # Create the cross plot trace
    cross_plot_trace5 = scatter(; 
        x=filtered_data5.CPIHeadline, 
		#y=filtered_GSCPI.GrowthOilPrices,
        y=filtered_data5.OilPrices, 
        mode="markers+lines", line_width = 0.3,
        marker=attr(color="blue", size=6.5), 
        name="",
		text=string.(filtered_data5.Months),  # Convert dates to strings for display
        hovertemplate="Inflation: %{x} <br> Growth oil prices: %{y}<br> %{text}"  # Custom hover info
    )

    # Create the figure with the cross plot trace
    fig5 = plot([cross_plot_trace5])

    # Customize layout
    relayout!(fig5,
		height = 550, width = 550,
        title_text="Inflation vs. Growth oil prices (2000-2024)",
        title_x=0.5,
        hovermode="closest",
        xaxis=attr(title="Inflation", showgrid=true, zeroline=false),
        yaxis=attr(title="Growth oil prices", zeroline=false)
    )

    # Display the figure
    fig5
end;

# ╔═╡ 3eaf8c0d-979a-473b-8576-ff11a8188c84
md"""
##### Supporting cells for exercise 11 (Sahm rule)
"""

# ╔═╡ b3d779f3-61e7-46ce-90f6-37c8275f1f5b
# Define a function to generate recession shading traces
    function recession_trace(dates, recession_data; fillcolor="gray") #fillcolor="rgba(255, 0, 0, 0.8)"
        # Function to find start and end dates of recession periods
        function find_recession_periods(dates, recession_data)
            periods = []
            start_idx = 0

            for i in 1:length(recession_data)
                if recession_data[i] == 1 && start_idx == 0
                    start_idx = i  # Start of recession
                elseif recession_data[i] == 0 && start_idx != 0
                    push!(periods, (dates[start_idx], dates[i-1]))  # End of recession
                    start_idx = 0
                end
            end

            # Handle if the series ends in a recession
            if start_idx != 0
                push!(periods, (dates[start_idx], dates[end]))
            end

            return periods
        end

        # Get recession periods from the data
        recession_periods = find_recession_periods(dates, recession_data)

        # Create scatter traces for shaded recession areas
        traces = []
        for period in recession_periods
            start = period[1]
            stop = period[2]
            trace = scatter(; x=[start, stop, stop, start], 
                            y=[-2, -2, 100, 100], 
                            fill="toself", 
                            fillcolor=fillcolor, 
                            opacity=0.3, 
                            line=attr(width=0), 
                            showlegend=false, 
                            name="Recession", 
                            hoverinfo="skip",  # Skips hover info for clean visuals
                            xaxis="x", 
                            yaxis="y")
            push!(traces, trace)
        end

        return traces
    end;

# ╔═╡ 057d6356-cfbe-43be-a3f5-434fcb8af3b5
recess = CSV.read("NBER_Recessions.csv", DataFrame);

# ╔═╡ b46e360d-c3ec-4fe5-a4f5-8ea6bd20b793
sahm = CSV.read("Sahm_Rule.csv", DataFrame);

# ╔═╡ c152cfce-d583-487b-bae5-f9444a393a10
begin

    # Example data (Replace with actual data)
    start_date = Date("1949-01-01")
    end_date = Date("2024-10-01")
    dates = collect(start_date:Month(1):end_date)
    recession_data = recess.Recessions[13:end]  # Replace with actual recession data

    # Get the recession traces
    recession_traces = recession_trace(dates, recession_data)

    # Example plot with another time series (e.g., unemployment)
    unemployment_data = sahm.UR  # Replace with actual data
    trace_unemployment = scatter(; x=dates, y=unemployment_data, mode="lines", line_color="blue", name="UR")

	sahm_data = sahm.SahmRule  # Replace with actual data
    trace_sahm = scatter(; x=dates, y=sahm_data, mode="lines", line_color="blue", name="Sahm")

    # Combine the recession traces with other data
    figx = plot([[trace_sahm]; recession_traces...])

    # Adjust layout
    relayout!(figx,
		height = 450,
        title_text="The Sahm rule",
        title_x=0.5,
        hovermode="x unified",
		xaxis_range = ["1947-12-01", "2024-12-01"],
		yaxis_range = [-0.5, 5],
        xaxis=attr(title="Date", showgrid=true, zeroline=false),
        yaxis=attr(title="Percentage", zeroline=false),

		annotations=[  attr(x="1996-09-01", y=0.51,
           					text="The Sahm Rule Trigger (0.5)",
							font=attr(
                					family="Calibri",
                					size=15,
                					color="black" ),
							ax=0, ay=-120,
            				showarrow=true,
            				arrowhead=3),
			
		 attr(x="1957-10-01", y=4.5, text="Shaded areas: NBER recessions", showarrow=false, yshift=10, 
			font=attr(family="Calibri",	size=15, color="black" ))
	
			]
    )

	add_shape!(figx, line( x0="1947-01-01", y0=0.5, x1="2024-12-01", y1=0.5 , line=attr(color="darkred", width=1.2)))

    # Display the figure
    figx
end

# ╔═╡ ad765d9e-4069-487c-9f70-a846539767f7
md"""
##### Supporting cells for exercise 12 (Policy coordination)
"""

# ╔═╡ bc297927-e93c-4e43-ad7b-74dd1c858056
CBrates = CSV.read("CentralBankRates.csv", DataFrame);

# ╔═╡ 53e45959-2372-4da9-8acc-47881157c6b6
begin
	fig33 = plot(CBrates.Months, [ CBrates.USi  CBrates.UKi  CBrates.EZi  CBrates.CANi])
	relayout!(fig33, height = 450, title_text ="Central Bank Rates (2005-2024)", title_x = 0.5, xaxis_range=["2005-01-01", "2024-09-01"], hovermode="x",
		
		annotations=[attr(x="2008-09-01", y=6, text="The great recession", showarrow=false, yshift=10, font=attr(
                					family="Calibri",
                					size=15,
                					color="black" )),

			attr(x="2020-07-01", y=6, text="Covid", showarrow=false, yshift=10, font=attr(
                					family="Calibri",
                					size=15,
                					color="black" )),
			attr(x="2023-06-01", y=6, text="Rampant inflation", showarrow=false, yshift=10, font=attr(
                					family="Calibri",
                					size=15,
                					color="black" )),
							
		]
	
	)
	restyle!(fig33, name = ["US" , "UK" , "EZ" , "CAN"], line_color =["blue", "red", "maroon", "purple"] )
	add_vrect!(fig33, "2007-08-01", "2009-04-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)
	add_vrect!(fig33, "2020-01-01", "2021-04-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)
	add_vrect!(fig33, "2022-01-01", "2024-09-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)
	fig33
end

# ╔═╡ 5ce0abd1-22e9-4f8a-aa7e-47d343188ab9
let
	fig33 = plot(CBrates.Months, [ CBrates.Usinf  CBrates.Ukinf  CBrates.Ezinf  CBrates.CANinf ])
	relayout!(fig33, height = 450, title_text ="Headline inflation", title_x = 0.5, xaxis_range=["2005-01-01", "2024-08-01"], hovermode="x")
	restyle!(fig33, name = ["US" , "UK" , "EZ" , "CAN"], line_color =["blue", "red", "maroon", "purple"] )
	add_vrect!(fig33, "2008-06-01", "2009-09-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)
	add_vrect!(fig33, "2020-01-01", "2020-12-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)
	add_vrect!(fig33, "2021-10-01", "2024-09-01",  fillcolor = "red", opacity = 0.2, line_width = 0; row=1, col=1)
	fig33
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
NLsolve = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
PeriodicalDates = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
StatsBase = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"

[compat]
CSV = "~0.10.15"
DataFrames = "~1.7.0"
HypertextLiteral = "~0.9.5"
NLsolve = "~4.5.1"
PeriodicalDates = "~2.0.0"
PlutoPlotly = "~0.6.2"
PlutoUI = "~0.7.48"
StatsBase = "~0.34.3"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.11.3"
manifest_format = "2.0"
project_hash = "1b712b90f0dcb2f73208fc330b3e3af2dae27986"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "6e1d2a35f2f90a4bc7c2ed98079b2ba09c35b83a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.2"

[[deps.Adapt]]
deps = ["LinearAlgebra", "Requires"]
git-tree-sha1 = "cd8b948862abee8f3d3e9b73a102a9ca924debb0"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "4.2.0"

    [deps.Adapt.extensions]
    AdaptSparseArraysExt = "SparseArrays"
    AdaptStaticArraysExt = "StaticArrays"

    [deps.Adapt.weakdeps]
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.AliasTables]]
deps = ["PtrArrays", "Random"]
git-tree-sha1 = "9876e1e164b144ca45e9e3198d0b689cadfed9ff"
uuid = "66dad0bd-aa9a-41b7-9441-69ab47430ed8"
version = "1.1.3"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.2"

[[deps.ArrayInterface]]
deps = ["Adapt", "LinearAlgebra"]
git-tree-sha1 = "017fcb757f8e921fb44ee063a7aafe5f89b86dd1"
uuid = "4fba245c-0d91-5ea0-9b3e-6abc04ee57a9"
version = "7.18.0"

    [deps.ArrayInterface.extensions]
    ArrayInterfaceBandedMatricesExt = "BandedMatrices"
    ArrayInterfaceBlockBandedMatricesExt = "BlockBandedMatrices"
    ArrayInterfaceCUDAExt = "CUDA"
    ArrayInterfaceCUDSSExt = "CUDSS"
    ArrayInterfaceChainRulesCoreExt = "ChainRulesCore"
    ArrayInterfaceChainRulesExt = "ChainRules"
    ArrayInterfaceGPUArraysCoreExt = "GPUArraysCore"
    ArrayInterfaceReverseDiffExt = "ReverseDiff"
    ArrayInterfaceSparseArraysExt = "SparseArrays"
    ArrayInterfaceStaticArraysCoreExt = "StaticArraysCore"
    ArrayInterfaceTrackerExt = "Tracker"

    [deps.ArrayInterface.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    CUDA = "052768ef-5323-5732-b1bb-66c8b64840ba"
    CUDSS = "45b445bb-4962-46a0-9369-b4df9d0f772e"
    ChainRules = "082447d4-558c-5d27-93f4-14fc19e9eca2"
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    GPUArraysCore = "46192b85-c4d5-4398-a991-12ede77f4527"
    ReverseDiff = "37e2e3b7-166d-5795-8a7a-e32c996b4267"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArraysCore = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
    Tracker = "9f7883ad-71c0-57eb-9f7f-b5c9e6d3789c"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.11.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"
version = "1.11.0"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "deddd8725e5e1cc49ee205a1964256043720a6c3"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.15"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "962834c22b66e32aa10f7611c08c8ca4e20749a9"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.8"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "403f2d8e209681fcbd9468a8514efff3ea08452e"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.29.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "b10d0b65641d57b8b4d5e234446582de5047050d"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.5"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"
weakdeps = ["SpecialFunctions"]

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "362a287c3aa50601b0bc359053d5c2468f0e7ce0"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.11"

[[deps.CommonSubexpressions]]
deps = ["MacroTools"]
git-tree-sha1 = "cda2cfaebb4be89c9084adaca7dd7333369715c5"
uuid = "bbf7d656-a473-5ed7-a52c-81e309532950"
version = "0.3.1"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "8ae8d32e09f0dcf42a36b90d4e17f5dd2e4c4215"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.16.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.1.1+0"

[[deps.ConstructionBase]]
git-tree-sha1 = "76219f1ed5771adbb096743bff43fb5fdd4c1157"
uuid = "187b0558-2788-49d3-abe0-74a17ed4e7c9"
version = "1.5.8"

    [deps.ConstructionBase.extensions]
    ConstructionBaseIntervalSetsExt = "IntervalSets"
    ConstructionBaseLinearAlgebraExt = "LinearAlgebra"
    ConstructionBaseStaticArraysExt = "StaticArrays"

    [deps.ConstructionBase.weakdeps]
    IntervalSets = "8197267c-284f-5f27-9208-e0e47529a953"
    LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "DataStructures", "Future", "InlineStrings", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrecompileTools", "PrettyTables", "Printf", "Random", "Reexport", "SentinelArrays", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "fb61b4812c49343d7ef0b533ba982c46021938a6"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.7.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1d0a14036acb104d9e89698bd408f63ab58cdc82"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.20"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"
version = "1.11.0"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DiffResults]]
deps = ["StaticArraysCore"]
git-tree-sha1 = "782dd5f4561f5d267313f23853baaaa4c52ea621"
uuid = "163ba53b-c6d8-5494-b064-1a9d43ac40c5"
version = "1.1.0"

[[deps.DiffRules]]
deps = ["IrrationalConstants", "LogExpFunctions", "NaNMath", "Random", "SpecialFunctions"]
git-tree-sha1 = "23163d55f885173722d1e4cf0f6110cdbaf7e272"
uuid = "b552c78f-8df3-52c6-915a-8e097449b14b"
version = "1.15.1"

[[deps.Distances]]
deps = ["LinearAlgebra", "Statistics", "StatsAPI"]
git-tree-sha1 = "c7e3a542b999843086e2f29dac96a618c105be1d"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.12"

    [deps.Distances.extensions]
    DistancesChainRulesCoreExt = "ChainRulesCore"
    DistancesSparseArraysExt = "SparseArrays"

    [deps.Distances.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"
version = "1.11.0"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates"]
git-tree-sha1 = "2ec417fc319faa2d768621085cc1feebbdee686b"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.23"
weakdeps = ["Mmap", "Test"]

    [deps.FilePathsBase.extensions]
    FilePathsBaseMmapExt = "Mmap"
    FilePathsBaseTestExt = "Test"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"
version = "1.11.0"

[[deps.FiniteDiff]]
deps = ["ArrayInterface", "LinearAlgebra", "Setfield"]
git-tree-sha1 = "f089ab1f834470c525562030c8cfde4025d5e915"
uuid = "6a86dc24-6348-571c-b903-95158fe2bd41"
version = "2.27.0"

    [deps.FiniteDiff.extensions]
    FiniteDiffBandedMatricesExt = "BandedMatrices"
    FiniteDiffBlockBandedMatricesExt = "BlockBandedMatrices"
    FiniteDiffSparseArraysExt = "SparseArrays"
    FiniteDiffStaticArraysExt = "StaticArrays"

    [deps.FiniteDiff.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "05882d6995ae5c12bb5f36dd2ed3f61c98cbb172"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.5"

[[deps.ForwardDiff]]
deps = ["CommonSubexpressions", "DiffResults", "DiffRules", "LinearAlgebra", "LogExpFunctions", "NaNMath", "Preferences", "Printf", "Random", "SpecialFunctions"]
git-tree-sha1 = "a2df1b776752e3f344e5116c06d75a10436ab853"
uuid = "f6369f11-7733-5829-9624-2563aa707210"
version = "0.10.38"

    [deps.ForwardDiff.extensions]
    ForwardDiffStaticArraysExt = "StaticArrays"

    [deps.ForwardDiff.weakdeps]
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"
version = "1.11.0"

[[deps.HashArrayMappedTries]]
git-tree-sha1 = "2eaa69a7cab70a52b9687c8bf950a5a93ec895ae"
uuid = "076d061b-32b6-4027-95e0-9a2c6f6d7e74"
version = "0.2.0"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "b6d6bfdd7ce25b0f9b2f6b3dd56b2673a66c8770"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.5"

[[deps.InlineStrings]]
git-tree-sha1 = "6a9fde685a7ac1eb3495f8e812c5a7c3711c2d5e"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.3"

    [deps.InlineStrings.extensions]
    ArrowTypesExt = "ArrowTypes"
    ParsersExt = "Parsers"

    [deps.InlineStrings.weakdeps]
    ArrowTypes = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
    Parsers = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"
version = "1.11.0"

[[deps.InvertedIndices]]
git-tree-sha1 = "6da3c4316095de0f5ee2ebd875df8721e7e0bdbe"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.1"

[[deps.IrrationalConstants]]
git-tree-sha1 = "e2222959fbc6c19554dc15174c81bf7bf3aa691c"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.4"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JLLWrappers]]
deps = ["Artifacts", "Preferences"]
git-tree-sha1 = "a007feb38b422fbdab534406aeca1b86823cb4d6"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.7.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.LaTeXStrings]]
git-tree-sha1 = "dda21b8cbd6a6c40d9d02a73230f9d70fed6918c"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.4.0"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.6.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"
version = "1.11.0"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.7.2+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"
version = "1.11.0"

[[deps.LineSearches]]
deps = ["LinearAlgebra", "NLSolversBase", "NaNMath", "Parameters", "Printf"]
git-tree-sha1 = "e4c3be53733db1051cc15ecf573b1042b3a712a1"
uuid = "d3d80556-e9d4-5f37-9878-2ab0fcc64255"
version = "7.3.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
version = "1.11.0"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "13ca9e2586b89836fd20cccf56e57e2b9ae7f38f"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.29"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"
version = "1.11.0"

[[deps.MIMEs]]
git-tree-sha1 = "1833212fd6f580c20d4291da9c1b4e8a655b128e"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "1.0.0"

[[deps.MacroTools]]
git-tree-sha1 = "72aebe0b5051e5143a079a4685a46da330a40472"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.15"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"
version = "1.11.0"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.6+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "ec4f7fbeab05d7747bdf98eb74d130a2a2ed298d"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.2.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"
version = "1.11.0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.12.12"

[[deps.NLSolversBase]]
deps = ["DiffResults", "Distributed", "FiniteDiff", "ForwardDiff"]
git-tree-sha1 = "a0b464d183da839699f4c79e7606d9d186ec172c"
uuid = "d41bc354-129a-5804-8e4c-c37616107c6c"
version = "7.8.3"

[[deps.NLsolve]]
deps = ["Distances", "LineSearches", "LinearAlgebra", "NLSolversBase", "Printf", "Reexport"]
git-tree-sha1 = "019f12e9a1a7880459d0173c182e6a99365d7ac1"
uuid = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
version = "4.5.1"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "cc0a5deefdb12ab3a096f00a6d42133af4560d71"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.1.2"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.27+1"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+2"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl"]
git-tree-sha1 = "1346c9208249809840c91b26703912dff463d335"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.6+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "cc4054e898b852042d7b503313f7ad03de99c3dd"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.8.0"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.PeriodicalDates]]
deps = ["Dates", "Printf", "RecipesBase"]
git-tree-sha1 = "e616941f8093e50a373e7d51875143213f82eca4"
uuid = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
version = "2.0.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "Random", "SHA", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.11.0"
weakdeps = ["REPL"]

    [deps.Pkg.extensions]
    REPLExt = "REPL"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Colors", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "90af5c9238c1b3b25421f1fdfffd1e8fca7a7133"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.20"

    [deps.PlotlyBase.extensions]
    DataFramesExt = "DataFrames"
    DistributionsExt = "Distributions"
    IJuliaExt = "IJulia"
    JSON3Ext = "JSON3"

    [deps.PlotlyBase.weakdeps]
    DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
    Distributions = "31c24e10-a181-5473-b8eb-7969acd0382f"
    IJulia = "7073ff75-c697-5162-941a-fcdaad2a7d2a"
    JSON3 = "0f8b85d8-7281-11e9-16c2-39a750bddbf1"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "Artifacts", "ColorSchemes", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "PrecompileTools", "Reexport", "ScopedValues", "Scratch", "TOML"]
git-tree-sha1 = "9ebe25fc4703d4112cc418834d5e4c9a4b29087d"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.6.2"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "7e71a55b87222942f0f9337be62e26b1f103d3e4"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.61"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "5aa36f7049a63a1528fe8f7c3f2113413ffd4e1f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.1"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9306f6085165d270f7e3db02af26a400d580f5c6"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.3"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "1101cd475833706e4d0e7b122218257178f48f34"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.4.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"
version = "1.11.0"

[[deps.PtrArrays]]
git-tree-sha1 = "1d36ef11a9aaf1e8b74dacc6a731dd1de8fd493d"
uuid = "43287f4e-b6f4-7ad1-bb20-aadabca52c3d"
version = "1.3.0"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "StyledStrings", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"
version = "1.11.0"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"
version = "1.11.0"

[[deps.RecipesBase]]
deps = ["PrecompileTools"]
git-tree-sha1 = "5c3d09cc4f31f5fc6af001c250bf1278733100ff"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.4"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "62389eeff14780bfe55195b7204c0d8738436d64"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.1"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.ScopedValues]]
deps = ["HashArrayMappedTries", "Logging"]
git-tree-sha1 = "1147f140b4c8ddab224c94efa9569fc23d63ab44"
uuid = "7e506255-f358-4e82-b7e4-beb19740aa63"
version = "1.3.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "3bac05bc7e74a75fd9cba4295cde4045d9fe2386"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.2.1"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "712fb0231ee6f9120e005ccd56297abbc053e7e0"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.8"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"
version = "1.11.0"

[[deps.Setfield]]
deps = ["ConstructionBase", "Future", "MacroTools", "StaticArraysCore"]
git-tree-sha1 = "c5391c6ace3bc430ca630251d02ea9687169ca68"
uuid = "efcf1570-3423-57d1-acb7-fd33fddbac46"
version = "1.1.2"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"
version = "1.11.0"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.11.0"

[[deps.SpecialFunctions]]
deps = ["IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "64cca0c26b4f31ba18f13f6c12af7c85f478cfde"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.5.0"

    [deps.SpecialFunctions.extensions]
    SpecialFunctionsChainRulesCoreExt = "ChainRulesCore"

    [deps.SpecialFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"

[[deps.StaticArraysCore]]
git-tree-sha1 = "192954ef1208c7019899fbf8049e717f92959682"
uuid = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
version = "1.4.3"

[[deps.Statistics]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "ae3bb1eb3bba077cd276bc5cfc337cc65c3075c0"
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.11.1"
weakdeps = ["SparseArrays"]

    [deps.Statistics.extensions]
    SparseArraysExt = ["SparseArrays"]

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.StatsBase]]
deps = ["AliasTables", "DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "29321314c920c26684834965ec2ce0dacc9cf8e5"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.34.4"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "725421ae8e530ec29bcbdddbe91ff8053421d023"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.4.1"

[[deps.StyledStrings]]
uuid = "f489334b-da3d-4c2e-b8f0-e476e12c162b"
version = "1.11.0"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.7.0+0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "598cd7c1f68d1e205689b1c2fe65a9f85846f297"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.12.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"
version = "1.11.0"

[[deps.TranscodingStreams]]
git-tree-sha1 = "0c45878dcfdcfa8480052b6ab162cdd138781742"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.11.3"

[[deps.Tricks]]
git-tree-sha1 = "6cae795a5a9313bbb4f60683f7263318fc7d1505"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.10"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"
version = "1.11.0"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"
version = "1.11.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.11.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.59.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─a0775632-4992-440b-b4b8-e7edfbf2ce39
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─55cbc248-6462-4242-b0e6-14ccc26c1777
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─416e4000-a14b-4517-a067-34a2ddaaa639
# ╟─6716e258-7411-43c0-811d-8884ae76e04c
# ╟─8080d79c-e64d-44f6-b949-5b54d5d5f7a4
# ╟─0bc2eb14-3b5a-4203-82dd-3ef2044a2c93
# ╟─5999a598-0408-44f1-bd33-c4d0fa5a6006
# ╟─5ceac821-7004-4764-b2ee-b9e2aa0f6fdf
# ╟─6f107fa8-4921-4b4b-bbee-6434ab5e9fb3
# ╟─450dec57-18a3-4031-bf36-36a422d0fd34
# ╟─eb74f385-1688-45ea-8ad1-11bce1e48f87
# ╟─c90e721d-f4ca-473c-8a7c-7159d66d81cf
# ╟─4b6af178-725d-4841-8d62-102c561428c6
# ╟─bd22c100-271d-4496-a55f-792f0f92117f
# ╟─058ec5a1-5e74-472e-8892-c0fad163638b
# ╟─21d4988a-e438-4562-bc8f-a1d847020e25
# ╟─8566ec00-329a-48a8-9b75-0b1a7b9ef9eb
# ╟─afeda154-0ce5-4e1b-9e12-24420dd72681
# ╟─08203cfc-1ede-432b-b81c-f0a5cbaa5816
# ╟─40913213-c3f7-4dd7-9ffa-c1390f80c099
# ╟─c49227b4-af6d-4c5b-a2fb-ffd15b1c84b8
# ╟─b6e384ad-f8fb-434c-944a-91ac9f729068
# ╟─a429634a-1e14-4ed5-bfd0-109b0f1448a8
# ╟─7e7af7e3-8b07-4ce7-871e-f31fae3c13a2
# ╟─f15cce59-c216-4c46-904e-bf1fafbdaa07
# ╟─ec759c26-25d0-4554-96ea-665f6b694204
# ╟─c38dbe18-a85a-4960-9c99-674196e0526d
# ╟─cb0d6593-5ca4-4afd-bddc-88506c4cf477
# ╟─35612933-4d9a-4b88-b40f-3a2eec3a20fa
# ╟─6b39a08d-9f0d-4e46-9f63-549d7eb15c05
# ╟─50ddafe5-0bd0-405b-bdc6-c990b0500661
# ╟─d72389b4-b317-4d1a-928a-c002822ed785
# ╟─f20bd50f-8096-4169-b4ed-bd107fa2537a
# ╟─add07024-b7eb-445d-b4f4-6a6757c2ecd3
# ╟─8e2203e6-6677-47b9-8032-e9c63c0f41bb
# ╟─0fe4de49-3687-4900-a691-334cca6dd3c6
# ╠═a77a222e-aba3-4ac6-a5b0-a7551e3ccce4
# ╟─74ee6d92-ef34-47ce-8d57-39d41474a11d
# ╟─3aba8dac-7fa0-4193-9f41-ff8d9a49230f
# ╟─40d80d26-e808-43bf-833e-d1bb07b57630
# ╠═1b2dba45-7fbc-4658-9a02-1c54511f650a
# ╟─fc550324-0ffd-4039-b92e-67f12b6b8c90
# ╟─6e92b539-a91d-410f-a895-8d4b51b5257b
# ╟─faaa89d4-9543-40a7-9a61-747e17eb3388
# ╟─7ae85983-ee8e-4dd3-a7bf-474aaf74863c
# ╟─00f73a3a-f9b7-46a6-aca8-5bf340a79280
# ╟─a54b4a9b-a752-454f-af38-87ec1115512a
# ╟─445f88ea-3c01-4939-a2bc-a1c139213b17
# ╟─9b54c17b-e835-4478-95ae-a9e2d5de9f59
# ╟─7f7ca57b-5774-4184-8026-32ad0f56cb39
# ╟─7e47b3bc-a45c-4d84-9a58-d4f01d6ba99f
# ╟─5c2ce460-bd7a-4355-8b11-d8e6896d973d
# ╟─704145a9-ef2e-48ad-bbe2-b6bc750d62fc
# ╟─13b03571-e332-46f7-83d7-4a41fbcca0b5
# ╟─0941b1d5-b354-4857-9264-fcd2744e03a6
# ╟─63fb8be4-f6ce-457b-8d0c-b359f959f5a6
# ╟─4b11d361-dab6-4440-b269-499b36737f0d
# ╟─979c128b-76ed-4c28-b3ad-ffbbac481414
# ╟─1072c4ef-b41f-4de8-a64d-3cdaecf8949d
# ╟─1d0765df-9ee8-4e65-a0d1-bc3e54940846
# ╠═0252b84b-8411-4145-81ee-3e90892d859f
# ╟─9cefbcc6-397b-48d9-810b-05937873184a
# ╟─3aca9c62-a1e3-4478-a8a5-29d19535ad75
# ╟─dcd13560-0f73-4aaa-b59a-b7961d8cdc63
# ╟─7f67d656-bb55-4097-ba49-10140f9b3502
# ╟─fe1e6cbc-db87-40ac-836d-68069615c133
# ╟─22fd1017-4015-407d-bd79-188a763e87a4
# ╟─1d36abf0-a3ea-45d1-9ef2-1a8a1fdc50b4
# ╟─f21a5ae9-7887-4cf1-83f9-f1f0a0b3d5b3
# ╟─7beabfcd-9cbb-4eac-8bdb-298b60f541ac
# ╟─28052403-f02e-41dc-ba74-e42d88f3d494
# ╟─70e7a7ef-d49d-417e-8f87-ed9870857b45
# ╟─db702f28-e3e7-4d14-9209-1cbe2917247b
# ╟─39b9f0db-8735-4d0d-8e75-384bf21c1bc8
# ╟─5859e1fa-f4ef-42b7-a3fd-5085cd64161c
# ╟─933a7956-1037-401e-a0cd-e42ec8eeec2d
# ╟─ec34cbc5-fda5-4792-adc8-3bb0d02a7f22
# ╟─83c030a3-28bc-419f-968b-62ce9ab0a1e4
# ╟─ab9daba1-be70-4e7f-8304-639db1c16d2c
# ╟─f331c8bc-ab24-4547-b9e9-f1d543916862
# ╟─79d2fb58-371b-4720-976c-d5f6a70e46c6
# ╟─a8bfdb49-6a0a-448a-b84e-b5040f8a40c9
# ╟─c20a2418-ef6f-486d-a88b-6345655bb449
# ╟─7c7b66f0-2136-49bf-9aa5-829d5a342549
# ╟─992942ea-85b0-480d-a872-ba668092f55c
# ╟─786fb701-2d51-4257-a474-39779b6b98e5
# ╟─07f23f65-74e4-4a0f-aa18-ad252c25035f
# ╟─56570fa6-7445-4858-a0be-845ef4a50510
# ╟─e6542ffd-3ca5-4bf2-a9e8-01bf8b5f1f17
# ╟─b8fe25b2-7995-4394-9b54-d515493525ec
# ╟─6cf60fca-ee40-4036-9bdc-4c080983658b
# ╟─909376d1-fe62-4823-8a03-916147ec6da3
# ╟─7695e6fb-92b5-4a8d-aea3-a4c9fb67bf8f
# ╟─48899ef5-f238-4a43-8589-1e0379a35a39
# ╟─92205224-29aa-4318-9f5d-6170a5d714d8
# ╟─1a322c48-9310-4850-826f-be458ea6de84
# ╟─4d52d8da-73fa-4001-988e-b50a01081295
# ╟─2052e846-8d58-46f5-89d5-1471929c9b4e
# ╟─3d1d85f7-f000-40bb-ad7a-6aaace47d8f5
# ╟─bbc0b205-f298-41e6-b216-3177389815e3
# ╟─765bcd30-07e8-4a38-ae79-30f513bbfc49
# ╟─27ec69d5-a112-48fb-ac46-9673f06d7322
# ╟─c152cfce-d583-487b-bae5-f9444a393a10
# ╟─2f6baaac-a510-452e-b8e0-b84294d39aae
# ╟─8dd55cb9-c3b5-4be6-b490-5f50a8d3250e
# ╟─84541347-f1fb-42b6-ba7b-74712c6def7d
# ╟─6c1dbd2e-5bcf-4cc3-9309-e305f22d9260
# ╟─e9d17d76-4bb2-4395-b56e-991c00a8d5c5
# ╟─d7527a4a-a4e4-40e3-8bb0-3dda5c133aa7
# ╟─867af7b9-5847-4b56-8a98-5995c3da8159
# ╟─f7853e38-66f3-457e-951f-01e04cda9ba0
# ╟─08fe04ec-464e-4d1a-a0ca-84125903f1d5
# ╟─53e45959-2372-4da9-8acc-47881157c6b6
# ╟─aaa23489-2459-44ef-acd4-cbf25ae1f7a3
# ╟─130e1429-87a6-4ead-aa31-3d8ecbea91e5
# ╟─0d7a7c6d-5a53-4c00-a726-0eabf26d0edd
# ╟─e35df418-41e9-4f78-97fc-f427f588fb1e
# ╟─5ce0abd1-22e9-4f8a-aa7e-47d343188ab9
# ╟─1e427799-fcd1-4cb5-a1a6-a5c21470e515
# ╟─468f94f8-a10c-4ea3-b795-1ea4f2f2a724
# ╟─df3e386a-66f1-4525-b09d-a605c24aff21
# ╟─4a1e9746-51d3-4981-b685-2d1545fb2a07
# ╟─15c7a983-f8d3-44a5-a08a-7426dcee7639
# ╟─e1ad4f38-80cf-4730-9f5e-1a0c021becff
# ╟─34f82c93-1bad-4635-8e5f-3e9960657429
# ╟─ca679141-76e6-4b59-bbcb-ecfb923e8765
# ╟─de32bd19-9ef4-45f2-96cd-3a9675d9527d
# ╠═9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─cc156e07-e234-4f21-ac6e-ddaff41b3579
# ╟─f842d504-2d12-4fcc-966c-fdb6416d1ec5
# ╟─6d02586e-d1a4-4628-8c81-339db95b898e
# ╟─60e502e1-3990-49a2-b8f6-8f2ad744bb01
# ╟─45c90777-16b2-4359-9afa-31fdaf23fd73
# ╟─99b637d3-af40-4132-aa5b-d0d386b8d11a
# ╟─0ed308d6-1457-4571-9898-d6fe23ca827f
# ╟─1f24f76d-6f4e-4031-8d62-41b0ec351a2c
# ╟─62b870d7-80a7-46ea-b784-579d5ad289c2
# ╟─43e8fd65-2d3a-41bc-a8d6-44cf412318ed
# ╟─955e1066-8468-4553-b834-f21116d4a594
# ╟─df2a5996-55a7-4883-ac2f-59d4cce2c129
# ╟─e1fc276a-db60-4de2-b21e-4e6bfcdbe6e2
# ╟─f029f20a-d065-4211-b71f-18c56bc41a7a
# ╟─a1b1866b-3179-4829-a0cf-9bdbdb913c8c
# ╟─2d8d028a-2709-4ba4-bf95-cf927187c67a
# ╟─2b1953d1-4bb8-42c6-98be-c353a44d8857
# ╟─c435e0de-1b92-4062-a30e-a1da56a91914
# ╟─01ce076b-3033-4f1b-acb7-1f775c4ed1e1
# ╟─f760841b-8a69-4aa4-a76b-edbf27f7de63
# ╟─88491aa6-3a00-4237-b8a7-a1385b69eccf
# ╟─659c291b-ec4a-46d8-90ed-73fd3ce3b7b9
# ╟─c01915e5-2cab-4760-b9a9-6e7bf4fed84b
# ╟─920ce92a-437c-4b23-b56b-bf6d810329ba
# ╟─bb3bb4b1-7bb3-4bfd-aeaf-b11703c050e8
# ╟─0aaafde6-ada0-45ab-a684-3593d6e8dc10
# ╟─b21af848-8878-4de6-8a96-4221962a8fb5
# ╟─ac39db85-3a1e-4f49-99f9-d12153545e71
# ╟─d9d9266f-e090-4462-b535-634bcb311365
# ╟─18cb3039-8cc6-4faa-ba5e-f8c6bca82318
# ╟─f1e64c63-b2eb-4ac6-8306-0801ccb77e24
# ╟─eaf16e5b-4008-4bfe-8b00-2148ae964d61
# ╟─3eaf8c0d-979a-473b-8576-ff11a8188c84
# ╟─b3d779f3-61e7-46ce-90f6-37c8275f1f5b
# ╟─057d6356-cfbe-43be-a3f5-434fcb8af3b5
# ╟─b46e360d-c3ec-4fe5-a4f5-8ea6bd20b793
# ╟─ad765d9e-4069-487c-9f70-a846539767f7
# ╟─bc297927-e93c-4e43-ad7b-74dd1c858056
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
